package hudson.plugins.starteam;

import hudson.model.AbstractBuild;
import hudson.scm.ChangeLogSet;

import java.util.Collections;
import java.util.Iterator;
import java.util.List;

/**
 * <p>
 * Implementation of {@link ChangeLogSet} for StarTeam SCM.
 * </p>
 * 
 * @author Eric D. Broyles
 * @version 1.0
 */
public class StarTeamChangeLogSet extends ChangeLogSet<StarTeamChangeLogEntry>
{

  private List<StarTeamChangeLogEntry> history = null;

  public StarTeamChangeLogSet(AbstractBuild<?, ?> build, List<StarTeamChangeLogEntry> logs)
  {
    super(build);
    this.history = Collections.unmodifiableList(logs);
  }

  @Override
  public boolean isEmptySet()
  {
    return history.isEmpty();
  }

  public Iterator<StarTeamChangeLogEntry> iterator()
  {
    return history.iterator();
  }

  /**
   * Return the history for this change log set.
   * 
   * @return
   */
  public List<StarTeamChangeLogEntry> getHistory()
  {
    return history;
  }

}
